@@ -1,5 +1,5 @@
 """
-Constants required for reading JCAMP files
+Constants required for reading JCAMP files.
 """
 
 ################################################################################
@@ -20,7 +20,9 @@ Constants required for reading JCAMP files
 #    along with this program; if not, write to the Free Software               #
 #    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 #
 #                                                                              #
+################################################################################
 
+__all__ = ["JcampTagWarning", "header_info_fields", "xydata_tags"]
 
 header_info_fields = [
 		"TITLE",
@@ -60,11 +62,20 @@ xydata_tags = {"XYDATA", "DATA TABLE", "XYPOINTS, PEAK TABLE", "PEAK TABLE", "XY
 
 
 class JcampTagWarning(UserWarning):
-	def __init__(self, tag):
+	"""
+	Warning emitted when an unrecognised Jcamp tag is detected.
+
+	:param tag: The name of the tag.
+	"""
+
+	#: The name of the tag.
+	tag: str
+
+	def __init__(self, tag: str):
 		self.tag = str(tag)
-	
-	def __repr__(self):
+
+	def __repr__(self) -> str:
 		return f"JcampTagWarning(Unrecognised tag. tag={self.tag})"
-	
-	def __str__(self):
-		return f"Unrecognised tag {self.tag}."
\ No newline at end of file
+
+	def __str__(self) -> str:
+		return f"Unrecognised tag {self.tag}."
