@@ -1,5 +1,5 @@
 """
-Custom JSON Encoder to support PyMassSpec classes
+Custom JSON Encoder to support PyMassSpec classes.
 """
 
 ################################################################################
@@ -28,14 +28,16 @@ import sdjson
 # this package
 from pyms.Spectrum import MassSpectrum, Scan
 
+__all__ = ["PyMassSpecEncoder"]
+
 
 class PyMassSpecEncoder(sdjson.JSONEncoder):
 	"""
-	Custom JSON Encoder to support PyMassSpec classes
-	
+	Custom JSON Encoder to support PyMassSpec classes.
+
 	.. note: Currently only supports Scan and MassSpectrum objects
 	"""
-	
+
 	def default(self, o):
 		if isinstance(o, (Scan, MassSpectrum)):
 			return dict(o)
